@@ -74,7 +74,8 @@ typedef void GIsiIndication (GIsiClient *client,
 		const void *restrict data, size_t len,
 		uint16_t object, void *opaque);
 
-typedef void GIsiVerify (GIsiClient *client, bool alive, void *opaque);
+typedef void GIsiVerify (GIsiClient *client, bool alive, uint16_t object,
+				void *opaque);
 
 typedef bool GIsiResponse(GIsiClient *client,
 			  void const * restrict data, size_t len,
@@ -1016,21 +1017,6 @@ static void isi_hangup(struct ofono_voicecall *ovc,
 			     cb, data);
 }
 
-static void isi_list_calls(struct ofono_voicecall *ovc,
-			   ofono_call_list_cb_t cb, void *data)
-{
-	struct isi_voicecall *ivc = ofono_voicecall_get_data(ovc);
-	struct ofono_call calls[7];
-	int n, i;
-
-	for (i = 1, n = 0; i <= 7; i++) {
-		if (ivc->calls[i].status != CALL_STATUS_IDLE)
-			calls[n++] = isi_call_as_ofono_call(&ivc->calls[i]);
-	}
-
-	CALLBACK_WITH_FAILURE(cb, n, calls, data);
-}
-
 static void isi_release_all_held(struct ofono_voicecall *ovc,
 			ofono_voicecall_cb_t cb, void *data)
 {
@@ -1315,7 +1301,7 @@ static int isi_voicecall_probe(struct ofono_voicecall *ovc,
 }
 
 static void isi_call_verify_cb(GIsiClient *client,
-			       bool alive,
+			       bool alive, uint16_t object,
 			       void *ovc)
 {
 	if (alive) {
@@ -1369,7 +1355,6 @@ static struct ofono_voicecall_driver driver = {
 	.dial			= isi_dial,
 	.answer			= isi_answer,
 	.hangup			= isi_hangup,
-	.list_calls		= isi_list_calls,
 	.hold_all_active	= isi_hold_all_active,
 	.release_all_held	= isi_release_all_held,
 	.set_udub		= isi_set_udub,
