@@ -375,8 +375,9 @@ static gboolean set_own_numbers(struct ofono_sim *sim,
 			sim_adn_build(efmsisdn, sim->efmsisdn_length,
 					number, NULL);
 			new_numbers = new_numbers->next;
-		} else
+		} else {
 			memset(efmsisdn, 0xff, sim->efmsisdn_length);
+		}
 
 		if (ofono_sim_write(req->sim, SIM_EFMSISDN_FILEID,
 				msisdn_set_cb, OFONO_SIM_FILE_STRUCTURE_FIXED,
@@ -1171,8 +1172,9 @@ skip_efpl:
 			sim->language_prefs = concat_lang_prefs(NULL, efpl);
 		else
 			sim->language_prefs = concat_lang_prefs(efli, efpl);
-	} else
+	} else {
 		sim->language_prefs = concat_lang_prefs(efpl, efli);
+	}
 
 	if (sim->efli) {
 		g_free(sim->efli);
@@ -1720,6 +1722,42 @@ void ofono_sim_set_ready(struct ofono_sim *sim)
 	}
 }
 
+static void sim_cb_download_cb(const struct ofono_error *error,
+				const unsigned char *data, int len, void *user)
+{
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		ofono_error("CellBroadcast download to UICC failed");
+		return;
+	}
+
+	ofono_debug("CellBroadcast download to UICC reported no error");
+}
+
+void __ofono_cbs_sim_download(struct ofono_sim *sim,
+				const guint8 *pdu, int pdu_len)
+{
+	guint8 tlv[pdu_len + 8];
+
+	if (sim->ready != TRUE)
+		return;
+
+	if (sim->driver->envelope == NULL)
+		return;
+
+	tlv[0] = 0xd2; /* Cell Broadcast Download */
+	tlv[1] = 6 + pdu_len;
+	tlv[2] = 0x82; /* Device Identities */
+	tlv[3] = 0x02; /* Device Identities length */
+	tlv[4] = 0x83; /* Network */
+	tlv[5] = 0x81; /* UICC */
+	tlv[6] = 0x8c; /* Cell Broadcast page */
+	tlv[7] = pdu_len;
+
+	memcpy(tlv + 8, pdu, pdu_len);
+
+	sim->driver->envelope(sim, pdu_len + 8, tlv, sim_cb_download_cb, sim);
+}
+
 int ofono_sim_driver_register(const struct ofono_sim_driver *d)
 {
 	DBG("driver: %p, name: %s", d, d->name);
