@@ -706,13 +706,9 @@ static gboolean real_emit_call_list_changed(void *data)
 
 static void emit_call_list_changed(struct ofono_voicecall *vc)
 {
-#ifdef DELAY_EMIT
 	if (vc->emit_calls_source == 0)
 		vc->emit_calls_source =
 			g_timeout_add(0, real_emit_call_list_changed, vc);
-#else
-	real_emit_call_list_changed(vc);
-#endif
 }
 
 static gboolean real_emit_multiparty_call_list_changed(void *data)
@@ -738,14 +734,9 @@ static gboolean real_emit_multiparty_call_list_changed(void *data)
 
 static void emit_multiparty_call_list_changed(struct ofono_voicecall *vc)
 {
-#ifdef DELAY_EMIT
 	if (vc->emit_multi_source == 0)
 		vc->emit_multi_source = g_timeout_add(0,
 				real_emit_multiparty_call_list_changed, vc);
-	}
-#else
-	real_emit_multiparty_call_list_changed(vc);
-#endif
 }
 
 static void voicecalls_release_queue(struct ofono_voicecall *vc, GSList *calls)
@@ -838,8 +829,9 @@ static ofono_bool_t clir_string_to_clir(const char *clirstr,
 	} else if (!strcmp(clirstr, "enabled")) {
 		*clir = OFONO_CLIR_OPTION_INVOCATION;
 		return TRUE;
-	} else
+	} else {
 		return FALSE;
+	}
 }
 
 static struct ofono_call *synthesize_outgoing_call(struct ofono_voicecall *vc,
@@ -854,7 +846,7 @@ static struct ofono_call *synthesize_outgoing_call(struct ofono_voicecall *vc,
 	if (!call)
 		return call;
 
-	call->id = __ofono_modem_alloc_callid(modem);
+	call->id = __ofono_modem_callid_next(modem);
 
 	if (call->id == 0) {
 		ofono_error("Failed to alloc callid, too many calls");
@@ -862,6 +854,8 @@ static struct ofono_call *synthesize_outgoing_call(struct ofono_voicecall *vc,
 		return NULL;
 	}
 
+	__ofono_modem_callid_hold(modem, call->id);
+
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &number,
 				DBUS_TYPE_INVALID) == FALSE)
 		number = "";
@@ -1473,7 +1467,9 @@ void ofono_voicecall_disconnected(struct ofono_voicecall *vc, int id,
 
 	ofono_debug("Got disconnection event for id: %d, reason: %d", id, reason);
 
-	l = g_slist_find_custom(vc->call_list, GINT_TO_POINTER(id),
+	__ofono_modem_callid_release(modem, id);
+
+	l = g_slist_find_custom(vc->call_list, GUINT_TO_POINTER(id),
 				call_compare_by_id);
 
 	if (!l) {
@@ -1487,7 +1483,7 @@ void ofono_voicecall_disconnected(struct ofono_voicecall *vc, int id,
 	ts = time(NULL);
 	prev_status = call->call->status;
 
-	l = g_slist_find_custom(vc->multiparty_list, GINT_TO_POINTER(id),
+	l = g_slist_find_custom(vc->multiparty_list, GUINT_TO_POINTER(id),
 				call_compare_by_id);
 
 	if (l) {
@@ -1504,8 +1500,6 @@ void ofono_voicecall_disconnected(struct ofono_voicecall *vc, int id,
 
 	vc->release_list = g_slist_remove(vc->release_list, call);
 
-	__ofono_modem_release_callid(modem, id);
-
 	if (reason != OFONO_DISCONNECT_REASON_UNKNOWN)
 		voicecall_emit_disconnect_reason(call, reason);
 
@@ -1531,12 +1525,12 @@ void ofono_voicecall_notify(struct ofono_voicecall *vc,
 	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
 	GSList *l;
 	struct voicecall *v = NULL;
-	struct ofono_call *newcall = NULL;
+	struct ofono_call *newcall;
 
 	ofono_debug("Got a voicecall event, status: %d, id: %u, number: %s",
 			call->status, call->id, call->phone_number.number);
 
-	l = g_slist_find_custom(vc->call_list, GINT_TO_POINTER(call->id),
+	l = g_slist_find_custom(vc->call_list, GUINT_TO_POINTER(call->id),
 				call_compare_by_id);
 
 	if (l) {
@@ -1550,33 +1544,27 @@ void ofono_voicecall_notify(struct ofono_voicecall *vc,
 
 	ofono_debug("Did not find a call with id: %d\n", call->id);
 
-	newcall = g_try_new0(struct ofono_call, 1);
+	__ofono_modem_callid_hold(modem, call->id);
 
-	if (!call) {
+	newcall = g_memdup(call, sizeof(struct ofono_call));
+
+	if (!newcall) {
 		ofono_error("Unable to allocate call");
-		goto err;
-	}
-
-	memcpy(newcall, call, sizeof(struct ofono_call));
-
-	if (__ofono_modem_alloc_callid(modem) != call->id) {
-		ofono_error("Warning: Call id and internally tracked id"
-				" do not correspond");
-		goto err;
+		goto error;
 	}
 
 	v = voicecall_create(vc, newcall);
 
 	if (!v) {
 		ofono_error("Unable to allocate voicecall_data");
-		goto err;
+		goto error;
 	}
 
 	v->detect_time = time(NULL);
 
 	if (!voicecall_dbus_register(v)) {
 		ofono_error("Unable to register voice call");
-		goto err;
+		goto error;
 	}
 
 	vc->call_list = g_slist_insert_sorted(vc->call_list, v, call_compare);
@@ -1585,7 +1573,7 @@ void ofono_voicecall_notify(struct ofono_voicecall *vc,
 
 	return;
 
-err:
+error:
 	if (newcall)
 		g_free(newcall);
 
@@ -1676,8 +1664,42 @@ static void set_new_ecc(struct ofono_voicecall *vc)
 	emit_en_list_changed(vc);
 }
 
-static void ecc_read_cb(int ok, int total_length, int record, const unsigned char *data,
-			int record_length, void *userdata)
+static void ecc_g2_read_cb(int ok, int total_length, int record,
+				const unsigned char *data,
+				int record_length, void *userdata)
+{
+	struct ofono_voicecall *vc = userdata;
+	char en[7];
+
+	DBG("%d", ok);
+
+	if (!ok)
+		return;
+
+	if (total_length < 3) {
+		ofono_error("Unable to read emergency numbers from SIM");
+		return;
+	}
+
+	total_length /= 3;
+	while (total_length--) {
+		extract_bcd_number(data, 3, en);
+		data += 3;
+
+		if (en[0] != '\0')
+			vc->new_en_list = g_slist_prepend(vc->new_en_list,
+								g_strdup(en));
+	}
+
+	if (vc->new_en_list == NULL)
+		return;
+
+	set_new_ecc(vc);
+}
+
+static void ecc_g3_read_cb(int ok, int total_length, int record,
+				const unsigned char *data,
+				int record_length, void *userdata)
 {
 	struct ofono_voicecall *vc = userdata;
 	int total;
@@ -1834,8 +1856,13 @@ static void sim_watch(struct ofono_atom *atom,
 		return;
 	}
 
-	ofono_sim_read(sim, SIM_EFECC_FILEID, OFONO_SIM_FILE_STRUCTURE_FIXED,
-			ecc_read_cb, vc);
+	/* Try both formats, only one or none will work */
+	ofono_sim_read(sim, SIM_EFECC_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+			ecc_g2_read_cb, vc);
+	ofono_sim_read(sim, SIM_EFECC_FILEID,
+			OFONO_SIM_FILE_STRUCTURE_FIXED,
+			ecc_g3_read_cb, vc);
 }
 
 void ofono_voicecall_register(struct ofono_voicecall *vc)
@@ -1889,3 +1916,14 @@ void *ofono_voicecall_get_data(struct ofono_voicecall *vc)
 {
 	return vc->driver_data;
 }
+
+int ofono_voicecall_get_next_callid(struct ofono_voicecall *vc)
+{
+	struct ofono_modem *modem;
+	if (vc == NULL || vc->atom == NULL)
+		return 0;
+
+	modem = __ofono_atom_get_modem(vc->atom);
+
+	return __ofono_modem_callid_next(modem);
+}
